﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Boats
{
    public partial class FormBoat : Form
    {
        DrawingBoat _boat;
        public DrawingBoat SelectedBoat { get; private set; }
        public FormBoat()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Метод установки данных
        /// </summary>
        private void SetData()
        {
            Random rnd = new Random();
            _boat.SetPosition(
                rnd.Next(10, 100),
                rnd.Next(10, 100),
                pictureBoxBoat.Width,
                pictureBoxBoat.Height
            );
            toolStripStatusLabelSpeed.Text = $"Скорость: {_boat.Boat.Speed}";
            toolStripStatusLabelWeight.Text = $"Вес: {_boat.Boat.Weight}";
            toolStripStatusLabelBodyColor.Text = $"Цвет: {_boat.Boat.BodyColor.Name}";
        }
        /// <summary>
        /// Метод прорисовки лодки
        /// </summary>
        private void Draw()
        {
            Bitmap bmp = new(pictureBoxBoat.Width, pictureBoxBoat.Height);
            Graphics g = Graphics.FromImage(bmp);
            _boat?.DrawTransport(g);
            pictureBoxBoat.Image = bmp;
        }
        /// <summary>
        /// Обработчик нажатия кнопки "Создать"
        /// </summary>
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            Random rnd = new Random();
            Color color = Color.FromArgb(rnd.Next(0, 255), rnd.Next(0, 255), rnd.Next(0, 255));
            ColorDialog dialog = new ColorDialog();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }
            _boat = new DrawingBoat(
                rnd.Next(100, 300),
                rnd.Next(1000, 3000),
                color
            );
            SetData();
            Draw();
        }
        /// <summary>
        /// Изменение размеров формы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_boat == null)
                return;

            string btnName = ((Button)sender).Name;

            switch (btnName)
            {
                case "ButtonUp":
                    {
                        _boat.MoveTransport(Direction.Up);
                    }
                    break;
                case "ButtonDown":
                    {
                        _boat.MoveTransport(Direction.Down);
                    }
                    break;
                case "ButtonLeft":
                    {
                        _boat.MoveTransport(Direction.Left);
                    }
                    break;
                case "ButtonRight":
                    {
                        _boat.MoveTransport(Direction.Right);
                    }
                    break;
                default:
                    break;
            }
            Draw();
        }
        /// <summary>
        /// Изменение размеров формы
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PictureBoxBoat_Resize(object sender, EventArgs e)
        {
            _boat?.ChangeBorders(pictureBoxBoat.Width, pictureBoxBoat.Height);
            Draw();
        }
        /// <summary>
        /// Обработка нажатия кнопки "Модификация"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateModificate_Click(object sender, EventArgs e)
        {
            Random rnd = new Random();

            // Предлагаем установить свой основной цвет
            Color color = Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256));
            ColorDialog dialog = new ColorDialog();
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                color = dialog.Color;
            }

            // Предлагаем установить свой дополнительный цвет
            Color dopColor = Color.FromArgb(rnd.Next(0, 256), rnd.Next(0, 256), rnd.Next(0, 256));
            ColorDialog dialogDop = new();
            if (dialogDop.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialogDop.Color;
            }

            _boat = new DrawingCatamaran(
                rnd.Next(100, 300),
                rnd.Next(1000, 3000),
                color,
                dopColor,
                Convert.ToBoolean(rnd.Next(0, 2)),
                Convert.ToBoolean(rnd.Next(0, 2))
            );

            SetData();
            Draw();
        }
        /// <summary>
        /// Обработка нажатия кнопки "Выбрать"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSelectBoat_Click(object sender, EventArgs e)
        {
            SelectedBoat = _boat;
            DialogResult = DialogResult.OK;
        }
    }
}
