﻿using System;
using System.CodeDom;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Boats
{
    /// <summary>
    /// Класс формы-конфигуратора
    /// </summary>
    public partial class FormBoatConfig : Form
    {
        /// <summary>
        /// Переменная - выбранная лодка
        /// </summary>
        DrawingBoat _boat = null;
        /// <summary>
        /// Событие
        /// </summary>
        private event BoatDelegate EventAddBoat;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBoatConfig()
        {
            InitializeComponent();
            panelRed.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelNavy.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (sender, e) => Close();
        }
        /// <summary>
        /// Отрисовка лодки
        /// </summary>
        private void DrawBoat()
        {
            Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
            Graphics g = Graphics.FromImage(bmp);
            _boat?.SetPosition(5, 5, pictureBoxObject.Width, pictureBoxObject.Height);
            _boat?.DrawTransport(g);
            pictureBoxObject.Image = bmp;
        }
        /// <summary>
        /// Добавление события
        /// </summary>
        /// <param name="e"></param>
        public void AddEvent(BoatDelegate e)
        {
            if (EventAddBoat == null)
            {
                EventAddBoat = new BoatDelegate(e);
            }
            else
            {
                EventAddBoat += e;
            }
        }
        /// <summary>
        /// Применяем соответствующий объект
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверяем тип получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Применяем необходимый объект
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _boat = new DrawingBoat((int)numericUpDownSpeed.Value,
                        (int)numericUpDownWeight.Value,
                        Color.Red
                    );
                    break;
                case "labelModifiedObject":
                    _boat = new DrawingCatamaran((int)numericUpDownSpeed.Value,
                        (int)numericUpDownWeight.Value,
                        Color.Red,
                        Color.Blue,
                        checkBoxBobbers.Checked,
                        checkBoxSail.Checked
                    );
                    break;
            }
            DrawBoat();
        }
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor,
                DragDropEffects.Move | DragDropEffects.Copy);
        }
        /// <summary>
        /// Проверка получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Принимаем основной цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null)
                return;
            _boat.Boat.BodyColor = (Color)e.Data.GetData(typeof(Color).ToString());
            DrawBoat();
        }
        /// <summary>
        /// Проверка получаемой информации
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelDopColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
        /// <summary>
        /// Принимаем дополнительный цвет
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LabelDopColor_DragDrop(object sender, DragEventArgs e)
        {
            if (_boat == null || _boat is not DrawingCatamaran)
                return;
            (_boat.Boat as EntityCatamaran).SetDopColor((Color)e.Data.GetData(typeof(Color).ToString()));
            DrawBoat();
        }
        /// <summary>
        /// Добавление лодки
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOk_Click(object sender, EventArgs e)
        {
            EventAddBoat?.Invoke(_boat);
            Close();
        }
    }
}
