﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Boats
{
    /// <summary>
    /// Класс океанической карты
    /// </summary>
    internal class OceanMap : AbstractMap
    {
        /// <summary>
        /// Количество мин воруг центральной мины
        /// </summary>
        private readonly int minesAroundCount = 6;
        /// <summary>
        /// Количество центральных мин
        /// </summary>
        private readonly int circlesCount = 12;
        /// <summary>
        /// Цвет участка закрытого
        /// </summary>
        private readonly Brush mineColor = Brushes.Silver;
        /// <summary>
        /// Цвет участка открытого
        /// </summary>
        private readonly Brush openColor = Brushes.Aqua;
        /// <summary>
        /// Метод для отрисовки закрытого участка карты
        /// </summary>
        /// <param name="g"></param>
        /// <param name="i"></param>
        /// <param name="j"></param>
        protected override void DrawBarrierPart(Graphics g, int i, int j)
        {
            g.FillRectangle(openColor, i * _size_x, j * _size_y, _size_x, _size_y);
            g.FillEllipse(mineColor, i * _size_x, j * _size_y, _size_x, _size_y);
            g.FillEllipse(Brushes.Black, i * _size_x + _size_x * 0.25f, j * _size_y + _size_y * 0.25f, _size_x * 0.5f, _size_y * 0.5f);
            g.FillEllipse(Brushes.Red, i * _size_x + _size_x * 0.375f, j * _size_y + _size_y * 0.375f, _size_x * 0.25f, _size_y * 0.25f);
        }
        /// <summary>
        /// Метод для отрисовки открытого участка карты
        /// </summary>
        /// <param name="g"></param>
        /// <param name="i"></param>
        /// <param name="j"></param>
        protected override void DrawWaterPart(Graphics g, int i, int j)
        {
            g.FillRectangle(openColor, i * _size_x, j * _size_y, _size_x, _size_y);
        }
        /// <summary>
        /// Метод генерации карты
        /// </summary>
        protected override void GenerateMap()
        {
            _map = new int[100, 100];
            _size_x = (float)_width / _map.GetLength(0);
            _size_y = (float)_height / _map.GetLength(1);
            int counter = 0;
            for (int i = 0; i < _map.GetLength(0); ++i)
            {
                for (int j = 0; j < _map.GetLength(1); ++j)
                {
                    _map[i, j] = _freeWater;
                }
            }
            // Отрисовка "мин"
            double radius = Math.Min(_size_x, _size_y);
            while (counter < circlesCount)
            {
                int x = _random.Next(0, 99);
                int y = _random.Next(0, 99);
                _map[y, x] = _barrier;
                for (int i = 0; i < minesAroundCount; i++)
                {
                    int row = (int)Math.Ceiling(radius * Math.Sin(Math.PI * 2 / minesAroundCount * i)) + y;
                    int col = (int)Math.Ceiling(radius * Math.Cos(Math.PI * 2 / minesAroundCount * i)) + x;

                    if (row > -1 && col > -1 && row < _map.GetLength(0) && col < _map.GetLength(1))
                    {
                        _map[row, col] = _barrier;
                    }
                }
                counter++;
            }
        }
    }
}
