﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Boats
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetBoatsGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly List<T> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество элементов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBoatsGeneric(int count)
        {
            _places = new List<T>();
            _maxCount = count;
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="boat">Добавляемая лодка</param>
        /// <returns></returns>
        public int Insert(T boat)
        {
            // Проверка на _maxCount
            if (Count == _maxCount)
                return -1;
            // Вставка в начало набора
            return Insert(boat, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="boat">Добавляемая лодка</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T boat, int position)
        {
            // Проверка позиции
            if (position < 0 || position >= _maxCount - 1)
                return -1;
            _places.Insert(position, boat);
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns>Возвращает удаляемый объект или null, если не удалось удалить</returns>
        public T Remove(int position)
        {
            // Проверка позиции
            if (position < 0 || position >= Count)
                return null;
            T boat = _places[position];
            _places.RemoveAt(position);
            return boat;
        }
        public T this[int position]
        {
            get
            {
                // Проверка позиции
                if (position < 0 || position >= Count)
                    return null;
                return _places[position];
            }
            set
            {
                // Проверка позиции
                if (position < 0 || position >= Count)
                    return;
                // Вставка по позиции
                _places[position] = value;
            }
        }
        /// <summary>
        /// Проход по набору до первого пустого
        /// </summary>
        /// <returns></returns>
        public IEnumerable<T> GetBoats()
        {
            foreach (var boat in _places)
            {
                if (boat != null)
                {
                    yield return boat;
                }
                else
                {
                    yield break;
                }
            }
        }
    }
}
